@@ -12,7 +12,6 @@ use globset::{Glob, GlobSet, GlobSetBuilder};
 use serde_derive::{Deserialize, Serialize};
 use toml::Value as Toml;
 
-use crate::highlighting::THEME_SET;
 use crate::theme::Theme;
 use errors::{bail, Error, Result};
 use utils::fs::read_file;
@@ -97,6 +96,7 @@ pub struct SerializedConfig<'a> {
     title: &'a Option<String>,
     description: &'a Option<String>,
     languages: HashMap<&'a String, &'a languages::LanguageOptions>,
+    default_language: &'a str,
     generate_feed: bool,
     feed_filename: &'a str,
     taxonomies: &'a [taxonomies::Taxonomy],
@@ -105,6 +105,7 @@ pub struct SerializedConfig<'a> {
 }
 
 impl Config {
+    // any extra syntax and highlight themes have been loaded and validated already by the from_file method before parsing the config
     /// Parses a string containing TOML to our Config struct
     /// Any extra parameter will end up in the extra field
     pub fn parse(content: &str) -> Result<Config> {
@@ -117,15 +118,6 @@ impl Config {
             bail!("A base URL is required in config.toml with key `base_url`");
         }
 
-        if config.markdown.highlight_theme != "css"
-            && !THEME_SET.themes.contains_key(&config.markdown.highlight_theme)
-        {
-            bail!(
-                "Highlight theme {} defined in config does not exist.",
-                config.markdown.highlight_theme
-            );
-        }
-
         languages::validate_code(&config.default_language)?;
         for code in config.languages.keys() {
             languages::validate_code(code)?;
@@ -165,7 +157,16 @@ impl Config {
         let path = path.as_ref();
         let content =
             read_file(path).map_err(|e| errors::Error::chain("Failed to load config", e))?;
-        Config::parse(&content)
+
+        let mut config = Config::parse(&content)?;
+        let config_dir = path
+            .parent()
+            .ok_or_else(|| Error::msg("Failed to find directory containing the config file."))?;
+
+        // this is the step at which missing extra syntax and highlighting themes are raised as errors
+        config.markdown.init_extra_syntaxes_and_highlight_themes(config_dir)?;
+
+        Ok(config)
     }
 
     /// Makes a url, taking into account that the base url might have a trailing slash
@@ -291,6 +292,7 @@ impl Config {
             title: &options.title,
             description: &options.description,
             languages: self.languages.iter().filter(|(k, _)| k.as_str() != lang).collect(),
+            default_language: &self.default_language,
             generate_feed: options.generate_feed,
             feed_filename: &options.feed_filename,
             taxonomies: &options.taxonomies,
@@ -330,7 +332,7 @@ pub fn merge(into: &mut Toml, from: &Toml) -> Result<()> {
 
 impl Default for Config {
     fn default() -> Config {
-        Config {
+        let mut conf = Config {
             base_url: DEFAULT_BASE_URL.to_string(),
             title: None,
             description: None,
@@ -355,7 +357,9 @@ impl Default for Config {
             search: search::Search::default(),
             markdown: markup::Markdown::default(),
             extra: HashMap::new(),
-        }
+        };
+        conf.add_default_language();
+        conf
     }
 }
 
@@ -675,4 +679,32 @@ output_dir = "docs"
         let config = Config::parse(config).unwrap();
         assert_eq!(config.output_dir, "docs".to_string());
     }
+
+    // TODO: Tests for valid themes; need extra scaffolding (test site) for custom themes.
+
+    #[test]
+    fn invalid_highlight_theme() {
+        let config = r#"
+[markup]
+highlight_code = true
+highlight_theme = "asdf"
+    "#;
+
+        let config = Config::parse(config);
+        assert_eq!(config.is_err(), true);
+    }
+
+    #[test]
+    fn invalid_highlight_theme_css_export() {
+        let config = r#"
+[markup]
+highlight_code = true
+highlight_themes_css = [
+  { theme = "asdf", filename = "asdf.css" },
+]
+    "#;
+
+        let config = Config::parse(config);
+        assert_eq!(config.is_err(), true);
+    }
 }
